/*
 Termin
*/

/*****************************************************
ThemeEditor 1.0 est un diteur de thmes au format XML
Copyright (C) 2000 David MASCLE et Damien BOUCHARD.
Auteurs: David MASCLE et Damien BOUCHARD.
******************************************************/

import java.util.*;
import java.io.*;
import java.text.*;
import javax.swing.JOptionPane;

/**********************************************************************
 <b>ColoringClass</b> est la classe se chargeant des coloriages.<br><b>
 ThemeEditor 1.0</b> est un diteur de thmes au format XML.<br>
 <br>Copyright &copy; 2000 David MASCLE et Damien BOUCHARD.<br>
 
 @version : 1.0
 @author : David MASCLE et Damien BOUCHARD
**********************************************************************/

public class ColoringClass implements ThemeEditorConstants, ColoringFileConstants
{
    private EmptyWordsMap ewMap = null;
    private XmlThemeParser xmlTP = null;
    private List themeList = null;
    private String[] inputFileNames = null;
    private String outputPreviewFileName = System.getProperty("user.dir") + System.getProperty("file.separator") + OUTPUT_PATH + System.getProperty("file.separator") + OUTPUT_FILE;
    private String outputFileName = "";
    private List wordsList = null;
    private int numberOfWords = 0;
    private int numberOfEmptyWords = 0;
    private int numberOfWordsColored = 0;
    private List tokensList = null;
    private String xmlFileName = "";
    private String htmlFileName = "";
    private boolean generateHtmlAndXml = false;

    /**
       Dfini une classe de coloriage.
       @param emptyWordsFileName le nom du fichier de mots vides.
       @param themesFileName le nom du fichier de thmes.
       @param inputFileNames le nom du ou des fichiers d'entre.
       @param outputFileName le nom du fichier de sortie (cas du lancement sur la ligne de commande, le mettre  vide dans le cas d'une interface garphique).
     */
    public ColoringClass(String emptyWordsFileName, String themesFileName, String[] inputFileNames, String outputFileName)
    {	
	this.inputFileNames = inputFileNames;
	if (outputFileName.trim().length() != 0)
	    {
		this.outputFileName = outputFileName.trim();
		generateHtmlAndXml = true;
	    }
	wordsList = new ArrayList();
	tokensList = new ArrayList();
	// chargement des mots vides
	ewMap = new EmptyWordsMap(emptyWordsFileName);
	// chargement des thmes
	themeList = new ArrayList();
	xmlTP = new XmlThemeParser(themeList);
	xmlTP.parse(themesFileName);
	xmlTP = null;
	Collections.sort(themeList);
    }

    /**
       Mthode ffectuant le coloriage.
    */
    public void color()
    {
	if (outputFileName.length() == 0)
	    {
		// utilis dans le cas d'une utilisation sous interface graphique
		if (JOptionPane.showConfirmDialog(null, "Souhaitez vous gnrer les fichiers de sortie HTML et XML pour ce coloriage ?", "Fichiers de sortie HTML et XML", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)
		    {
			String userAnswer;
			boolean stillAsking = true;
			generateHtmlAndXml = true;
			while (xmlFileName.equals("") && stillAsking)
			    {
				
				userAnswer = JOptionPane.showInputDialog(null, "Veuillez saisir le nom qui sera attribu aux fichiers de sortie HTML et XML :", "Fichiers de sortie HTML et XML", JOptionPane.QUESTION_MESSAGE);
				if (userAnswer == null)
				    {
					stillAsking = false;
					generateHtmlAndXml = false;
					xmlFileName = "";
				    }		      
				else xmlFileName = userAnswer.trim();
			    }	     
		    }
		else
		    {
			generateHtmlAndXml = false;
		    }
	    }

	//******************************************
	// Dcomposition en units lexicales de base
	//******************************************
	ListOfFiles mylist = new ListOfFiles(inputFileNames);

       	Reader r = null;

	r = new BufferedReader(new InputStreamReader(new SequenceInputStream(mylist)));
	
	try
	    {
		MyStreamTokenizer myST = new MyStreamTokenizer(r);
		String token = null;
		
		while (myST.nextToken() != myST.TT_EOF)
		    {
			if (myST.ttype == myST.TT_WORD)
			    {
				token = myST.sval;
				tokensList.add(token);
			    }
			else if (myST.ttype == myST.TT_NUMBER)
			    {
				token = (new Double(myST.nval)).toString();
				tokensList.add(token);
			    }
			else if (myST.ttype == myST.TT_EOL)
			    {
				tokensList.add("");
			    }
			else tokensList.add(new Character((char)myST.ttype).toString());
		    }
	    }
	catch (Exception e)
	    {
		System.err.println("Erreur de tokenization du ou des  fichiers : ");
		for (int indf = 0; indf < inputFileNames.length; indf++)
		    System.err.println(" ==> " + inputFileNames[indf]);
		System.err.println("    " +  e.getLocalizedMessage());
	    }

	try
	    {
		r.close();
		r = null;
	    }
	catch (IOException e)
	    {
		System.err.println("Erreur d'entre/sortie sur le ou les fichiers : ");
		for (int indf = 0; indf < inputFileNames.length; indf++)
		    System.err.println(" ==> " + inputFileNames[indf]);
		System.err.println("    " +  e.getLocalizedMessage());
	    }
	
	//**********
	// Coloriage
	//**********
	int ind = 0;
	int MAX_TOKENS = 7; // dfini une fenetre de recherche compos de 4 mots
	int pos = 0;
	boolean wordAdded = false;
	while (ind < tokensList.size())
	    {
		if ((ind + MAX_TOKENS) < tokensList.size())
		    pos = ind + MAX_TOKENS;
		else pos = tokensList.size() - 1;
		wordAdded = false;
		
		while (!wordAdded)
		    {
			String myWord = "";
			
			for (int wi = ind; wi <= pos; wi++)
			    {
				myWord = myWord + (String)tokensList.get(wi);
				//System.out.println(myWord + "<br>");
			    }
			//System.out.println("pos = " + pos + ", " + myWord + "<br>");
			
			
			if (ewMap.containsWord(myWord))
			    {
				numberOfWords++;
				numberOfEmptyWords++;
				wordsList.add(new EmptyWord(myWord));
				wordAdded = true;
				ind = pos + 1;
			    }
			else
			    {
				ColoredWord cwTemp = null;
				cwTemp = getWordColor(myWord);
				if (cwTemp == null)
				    { 
					if (pos == ind)
					    {
						if (myWord.length() > 1)
						    {
							numberOfWords++;
						    }
						else if (myWord.length() == 1)
						    {
							int charValue = (int)myWord.charAt(0);
							if (((charValue >= 48) && (charValue <= 57)) || ((charValue >= 65) && (charValue <= 90)) || ((charValue >= 97) && (charValue <= 122)) || ((charValue >= 192) && (charValue <= 214)) || ((charValue >= 217) && (charValue <= 221)) || ((charValue >= 224) && (charValue <= 246)) || ((charValue >= 249) && (charValue <= 253)) || ((charValue >= 255) && (charValue <= 255)))
							    {
								numberOfWords++;
							    }
						    }
						if (myWord.length() == 0)
						    myWord = "<br>";
						wordsList.add(new StandardWord(myWord));
						wordAdded = true;
						ind = pos + 1;
					    }
				    }
				else
				    {
					numberOfWords++;
					numberOfWordsColored++;
					wordsList.add(cwTemp);
					wordAdded = true;
					ind = pos + 1;
				    }
			    }
			
			if (!wordAdded)
			    pos--;
		    }
	    }
	tokensList = null;
	ewMap = null;

	//************************************************
	// gnration des fichiers de sortie  du coloriage
	//************************************************
	generateOutputs();
    }

    /**
       Mthode gnrant les fichiers de sortie.
    */
    private void generateOutputs()
    {
	PrintWriter outToHtmlPreview = null;
	PrintWriter outToHtml = null;
	PrintWriter outToXml = null;
	
	if (outputFileName.length() == 0)
	    {
		try
		    {
			outToHtmlPreview = new PrintWriter(new BufferedWriter(new FileWriter(outputPreviewFileName)));
		    }
		catch (IOException e)
		    {
			System.err.println("Erreur d'entre/sortie sur le fichier de sortie : " + outputPreviewFileName);
			System.err.println("    " +  e.getLocalizedMessage());
		    }
	    }
	
	if (generateHtmlAndXml)
	    {
		// fichier html
		try
		    {
			if (outputFileName.length() == 0)
			    htmlFileName = System.getProperty("user.dir") + System.getProperty("file.separator") + OUTPUT_PATH + System.getProperty("file.separator") + xmlFileName + ".html";
			else
			    htmlFileName = System.getProperty("user.dir") + System.getProperty("file.separator") + outputFileName + ".html";
			
			outToHtml = new PrintWriter(new BufferedWriter(new FileWriter(htmlFileName)));
			System.out.println(htmlFileName);
		    }
		catch (IOException e)
		    {
			System.err.println("Erreur d'entre/sortie sur le fichier de sortie html : " + htmlFileName);
			System.err.println("    " +  e.getLocalizedMessage());
		    }
		// fichier xml
		try
		    {
			if (outputFileName.length() == 0)
			    xmlFileName = System.getProperty("user.dir") + System.getProperty("file.separator") + OUTPUT_PATH + System.getProperty("file.separator") + xmlFileName + ".xml";
			else
			    xmlFileName = System.getProperty("user.dir") + System.getProperty("file.separator") + outputFileName + ".xml";
			outToXml = new PrintWriter(new BufferedWriter(new FileWriter(xmlFileName)));
			System.out.println(xmlFileName);
		    }
		catch (IOException e)
		    {
			System.err.println("Erreur d'entre/sortie sur le fichier de sortie xml : " + xmlFileName);
			System.err.println("    " +  e.getLocalizedMessage());
		    }
	    }
	
	if (inputFileNames.length > 0)
	    {
		if (outputFileName.length() == 0)
		    {
			if (inputFileNames.length == 1)
			    outToHtmlPreview.println("<html><body bgcolor=\"#ffffff\"><h1 align=\"center\">Coloriage du texte : </h1>");
			else
			    outToHtmlPreview.println("<html><body bgcolor=\"#ffffff\"><h1 align=\"center\">Coloriage des textes : </h1>");
		    }
		
		if (generateHtmlAndXml)
		    {
			// fichier html
			if (inputFileNames.length == 1)
			    outToHtml.println("<html><body bgcolor=\"#ffffff\"><h1 align=\"center\">Coloriage du texte : </h1>");
			else
			    outToHtml.println("<html><body bgcolor=\"#ffffff\"><h1 align=\"center\">Coloriage des textes : </h1>");
			// fichier xml
			outToXml.println(COLORING_FILE_PROLOG + COLORING_FILE_DTD);
			outToXml.println("<" + COLORING_FILE_TAG + ">");
			outToXml.println("\t<" + LIST_OF_FILES_TAG + ">");
		    }
		
		for (int indf = 0; indf < inputFileNames.length; indf++)
		    {
			if (outputFileName.length() == 0)
			    outToHtmlPreview.println("<h2 align=\"center\">" + inputFileNames[indf] + "</h2>");
			if (generateHtmlAndXml)
			    {
				// fichier html
				outToHtml.println("<h2 align=\"center\">" + inputFileNames[indf] + "</h2>");
				// fichier xml
				outToXml.println("\t\t<" + FILE_TAG + ">" + inputFileNames[indf] + "</" + FILE_TAG + ">");
			    }
		    }
		
		if (generateHtmlAndXml)
		    {
			// fichier html
			outToHtml.println("<hr>");
			// fichier xml
			outToXml.println("\t</" + LIST_OF_FILES_TAG + ">");
			outToXml.println("\t<" + COLORING_TAG + ">");
		    }
		
		if (outputFileName.length() == 0)
		    outToHtmlPreview.println("<hr>");
		
		ListIterator i = wordsList.listIterator();
		while (i.hasNext())
		    {
			StandardWord sw = (StandardWord) i.next();
			if (outputFileName.length() == 0)
			    outToHtmlPreview.print(sw.toHtmlPreview());
			if (generateHtmlAndXml)
			    {
				// fichier html
				outToHtml.print(sw.toHtml());
				// fichier xml
				outToXml.println("\t\t" + sw.toXml());
			    }
		    }
		
		if (generateHtmlAndXml)
		    {
			// fichier html
			outToHtml.println("<br><hr>");
			// fichier xml
			outToXml.println("\t</" + COLORING_TAG + ">");
			outToXml.println("\t<" + STATISTICS_TAG + ">");
		    }
		
		wordsList = null;
		if (outputFileName.length() == 0)
		    outToHtmlPreview.println("<br><hr>");
		
		// affichage des statistiques
		if (outputFileName.length() == 0)
		    outToHtmlPreview.println("<h1 align=\"center\">Statistiques</h1>");
		
		// affichage du nombre de mots
		if (outputFileName.length() == 0)
		    outToHtmlPreview.println("<table align=\"center\" border=\"1\" cellpadding=\"2\" cellspacing=\"2\"><tr><td><b>Nombre de mots : </b></td><td align=\"right\">" + numberOfWords +"</td></tr></table><br>");
		if (generateHtmlAndXml)
		    {
			//*************
			// fichier html
			//*************
			outToHtml.println("<h1 align=\"center\">Statistiques</h1>");
		
			// affichage du nombre de mots
			outToHtml.println("<table align=\"center\" border=\"1\" cellpadding=\"2\" cellspacing=\"2\"><tr><td><b>Nombre de mots : </b></td><td align=\"right\">" + numberOfWords +"</td></tr></table><br>");

			//************
			// fichier xml
			//************
			outToXml.println("\t\t<" + NB_WORDS_TAG + ">" + numberOfWords + "</" + NB_WORDS_TAG + ">");
		    }
		// affichage mots vides et mots coloris

		if (outputFileName.length() == 0)
		    outToHtmlPreview.println("<table align=\"center\" border=\"1\" cellpadding=\"2\" cellspacing=\"2\"><tr><td></td><td align=\"center\"><b>Nombre</b></td><td align=\"center\"><b>Pourcentage</b></td></tr><tr><td><b>Mots colori&eacute;s : </b></td><td align=\"right\">" + numberOfWordsColored + "</td><td align=\"right\">" + calculatePercentage(numberOfWordsColored, numberOfWords) + " %</td></tr><tr><td><b>Mots vides : </b></td><td align=\"right\">" + numberOfEmptyWords + "</td><td align=\"right\">" + calculatePercentage(numberOfEmptyWords, numberOfWords) + " %</td></tr></table><br>");

		if (generateHtmlAndXml)
		    {
			//*************
			// fichier html
			//*************
			outToHtml.println("<table align=\"center\" border=\"1\" cellpadding=\"2\" cellspacing=\"2\"><tr><td></td><td align=\"center\"><b>Nombre</b></td><td align=\"center\"><b>Pourcentage</b></td></tr><tr><td><b>Mots colori&eacute;s : </b></td><td align=\"right\">" + numberOfWordsColored + "</td><td align=\"right\">" + calculatePercentage(numberOfWordsColored, numberOfWords) + " %</td></tr><tr><td><b>Mots vides : </b></td><td align=\"right\">" + numberOfEmptyWords + "</td><td align=\"right\">" + calculatePercentage(numberOfEmptyWords, numberOfWords) + " %</td></tr></table><br>");

			//************
			// fichier xml
			//************
			outToXml.println("\t\t<" + COLORED_WORDS_STATS_TAG + ">");
			outToXml.println("\t\t\t<" + NUMBER_TAG + ">" + numberOfWordsColored + "</" + NUMBER_TAG + ">");
			outToXml.println("\t\t\t<" + PERCENTAGE_TAG + ">" + calculatePercentage(numberOfWordsColored, numberOfWords) + "</" + PERCENTAGE_TAG + ">");
			outToXml.println("\t\t</" + COLORED_WORDS_STATS_TAG + ">");
			
			outToXml.println("\t\t<" + EMPTY_WORDS_STATS_TAG + ">");
			outToXml.println("\t\t\t<" + NUMBER_TAG + ">" + numberOfEmptyWords + "</" + NUMBER_TAG + ">");
			outToXml.println("\t\t\t<" + PERCENTAGE_TAG + ">" + calculatePercentage(numberOfEmptyWords, numberOfWords) + "</" + PERCENTAGE_TAG + ">");
			outToXml.println("\t\t</" + EMPTY_WORDS_STATS_TAG + ">");
		    }
		// affichage des thmes

		if (outputFileName.length() == 0)
		    outToHtmlPreview.println("<table align=\"center\" border=\"1\" cellpadding=\"2\" cellspacing=\"2\"><tr><td align=\"center\"><b>Classement</b></td><td align=\"center\"><b>D&eacute;finition du th&egrave;me</b></td><td align=\"center\"><b>Nombre de mots appartenant au th&egrave;me</b></td><td align=\"center\"><b>Couleur</b></td><td align=\"center\"><b>Nombre de mots colori&eacute;s</b></td><td align=\"center\"><b>Nombre de mots diff&eacute;rents utilis&eacute;s pour colorier</b></td><td align=\"center\"><b>Pourcentage de recouvrement du th&egrave;me</b></td><td align=\"center\"><b>Pourcentage (par rapport au nombre de mots colori&eacute;s)</b></td><td align=\"center\"><b>Pourcentage (par rapport au nombre de mots)</b></td></tr>");

		if (generateHtmlAndXml)
		    {
			// fichier html
			outToHtml.println("<table align=\"center\" border=\"1\" cellpadding=\"2\" cellspacing=\"2\"><tr><td align=\"center\"><b>Classement</b></td><td align=\"center\"><b>D&eacute;finition du th&egrave;me</b></td><td align=\"center\"><b>Nombre de mots appartenant au th&egrave;me</b></td><td align=\"center\"><b>Couleur</b></td><td align=\"center\"><b>Nombre de mots colori&eacute;s</b></td><td align=\"center\"><b>Nombre de mots diff&eacute;rents utilis&eacute;s pour colorier</b></td><td align=\"center\"><b>Pourcentage de recouvrement du th&egrave;me</b></td><td align=\"center\"><b>Pourcentage (par rapport au nombre de mots colori&eacute;s)</b></td><td align=\"center\"><b>Pourcentage (par rapport au nombre de mots)</b></td></tr>");

			// fichier xml
			outToXml.println("\t<" + THEMES_STATS_TAG + ">");
		    }

		ListIterator iter = themeList.listIterator();
		int rank = 1;
		boolean stillContinue = true;
		while (iter.hasNext() && stillContinue)
		    {
			Theme t = (Theme)iter.next();
			if (t.getNumberOfWordsColored() > 0)
			    {
				int numberOfWordsUsedToColor = t.getNumberOfWordsUsedToColor();
				if (outputFileName.length() == 0)
				    outToHtmlPreview.println("<tr><td>" + rank + "</td><td>" + t.getDefinition() + "</td><td>" + t.getNumberOfWordsInTheTheme() + "</td><td bgcolor=\"" + t.getColor() + "\">&nbsp;&nbsp;&nbsp;</td><td align=\"right\">" + t.getNumberOfWordsColored() + "</td><td align=\"right\">" + numberOfWordsUsedToColor + "</td><td align=\"right\">" + calculatePercentage(numberOfWordsUsedToColor, t. getNumberOfWordsInTheTheme()) + " %</td><td align=\"right\">" + calculatePercentage(t.getNumberOfWordsColored(), numberOfWordsColored) + " %</td><td align=\"right\">" + calculatePercentage(t.getNumberOfWordsColored(), numberOfWords) + " %</td></tr>");
				
				if (generateHtmlAndXml)
				    {
					// fichier html
					outToHtml.println("<tr><td>" + rank + "</td><td>" + t.getDefinition() + "</td><td>" + t.getNumberOfWordsInTheTheme() + "</td><td bgcolor=\"" + t.getColor() + "\">&nbsp;&nbsp;&nbsp;</td><td align=\"right\">" + t.getNumberOfWordsColored() + "</td><td align=\"right\">" + numberOfWordsUsedToColor + "</td><td align=\"right\">" + calculatePercentage(numberOfWordsUsedToColor, t. getNumberOfWordsInTheTheme()) + " %</td><td align=\"right\">" + calculatePercentage(t.getNumberOfWordsColored(), numberOfWordsColored) + " %</td><td align=\"right\">" + calculatePercentage(t.getNumberOfWordsColored(), numberOfWords) + " %</td></tr>");

					// fichier xml
					outToXml.println("\t\t<" + THEME_STATS_TAG + ">");
					outToXml.println("\t\t\t<" + RANK_TAG + ">" + rank + "</" + RANK_TAG + ">");
					outToXml.println("\t\t\t<" + DEFINITION_TAG + ">" + t.getDefinition() + "</" + DEFINITION_TAG + ">");
					outToXml.println("\t\t\t<" + NB_WORDS_IN_THE_THEME_TAG + ">" + t.getNumberOfWordsInTheTheme() + "</" + NB_WORDS_IN_THE_THEME_TAG + ">");
					outToXml.println("\t\t\t<" + COLOR_TAG + ">" + t.getColor() + "</" + COLOR_TAG + ">");
					outToXml.println("\t\t\t<" + NB_WORDS_COLORED_TAG + ">" + t.getNumberOfWordsColored() + "</" + NB_WORDS_COLORED_TAG + ">");
					outToXml.println("\t\t\t<" + NB_WORDS_USED_TO_COLOR_TAG + ">" + numberOfWordsUsedToColor + "</" + NB_WORDS_USED_TO_COLOR_TAG + ">");
					outToXml.println("\t\t\t<" + THEME_RECOVERING_PERCENTAGE_TAG + ">" + calculatePercentage(numberOfWordsUsedToColor, t. getNumberOfWordsInTheTheme()) + "</" + THEME_RECOVERING_PERCENTAGE_TAG + ">");
					outToXml.println("\t\t\t<" + PERCENTAGE_FOR_COLORED_WORDS_TAG + ">" + calculatePercentage(t.getNumberOfWordsColored(), numberOfWordsColored) + "</" + PERCENTAGE_FOR_COLORED_WORDS_TAG + ">");
					outToXml.println("\t\t\t<" + PERCENTAGE_FOR_ALL_WORDS_TAG + ">" + calculatePercentage(t.getNumberOfWordsColored(), numberOfWords) + "</" + PERCENTAGE_FOR_ALL_WORDS_TAG + ">");
					outToXml.println("\t\t</" + THEME_STATS_TAG + ">");
				    }
			    }
			else stillContinue = false;
			rank++;
		    }
		
		if (generateHtmlAndXml)
		    {
			// fichier html
			outToHtml.println("</table><br>");
			
			// fichier xml
			outToXml.println("\t</" + THEMES_STATS_TAG + ">");
		    }
		
		if (outputFileName.length() == 0)
		    {
			outToHtmlPreview.println("</table><br>");
			outToHtmlPreview.println("</body></html>");
		    }

		if (generateHtmlAndXml)
		    {
			// fichier html
			outToHtml.println("</body></html>");
			
			// fichier xml
			outToXml.println("\t</" + STATISTICS_TAG + ">");
			outToXml.println("</" + COLORING_FILE_TAG + ">");
		    }
	    }
	else if (outputFileName.length() == 0)
	    outToHtmlPreview.println("<html><body bgcolor=\"#ffffff\"><h1 align=\"center\">Erreur : aucun fichier  colorier !</h1></body></html>");
	
	if (generateHtmlAndXml)
	    {
		outToHtml.flush();
		outToHtml.close();
		outToXml.flush();
		outToXml.close();
	    }

	if (outputFileName.length() == 0)
	    {
		outToHtmlPreview.flush();
		outToHtmlPreview.close();
	    }
    }

    /**
       Mthode permettant le coloriage d'un mot.
       @parma word un mot.
       @return un mot colori.
    */
    private ColoredWord getWordColor(String word)
    {
	boolean found = false;
	ColoredWord cwTemp = null;
	ListIterator i = themeList.listIterator();
	while ((!found) && (i.hasNext()))
	    {
		Theme t = (Theme) i.next();
		if (t.containsWord(word))
		    {
			found = true;
			t.incrementNumberOfWordsColored();
			t.incrementNumberOfWordsColoredWith(word);
			cwTemp = new ColoredWord(word + " ", t.getColor());
		    }
	    }
	Collections.sort(themeList, Collections.reverseOrder());
	return cwTemp;
    }

    /**
       Mthode utilitaire de calcul et de formattage d'un pourcentage
    */
    private String calculatePercentage(int num, int den)
    {
	String result = "erreur";
	if (den >0)
	    {
		DecimalFormat dFormat = new DecimalFormat("0.00");
		result = dFormat.format(((double)num / den) * 100);
	    }
	return result;
    }
}
