/*
  Termin
*/

/******************************************************
ThemeEditor 1.0 est un diteur de thmes au format XML.
Copyright (C) 2000 David MASCLE et Damien BOUCHARD.
Auteurs: David MASCLE et Damien BOUCHARD.
*******************************************************/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;

/******************************************************************************
<b>EmptyWordsPanel</b> est un panel servant  l'dition des mots vides.<br><br>
<br><b>ThemeEditor 1.0</b> est un diteur de thmes au format XML.<br>
<br>Copyright &copy; 2000 David MASCLE et Damien BOUCHARD.<br>

@version : 1.0
@author : David MASCLE et Damien BOUCHARD

*******************************************************************************/

public class EmptyWordsPanel extends JPanel implements ListSelectionListener, ThemeEditorConstants
{
    private JList list;
    private DefaultListModel listModel;
    private static final String addString = "Ajouter";
    private static final String modifyString = "Modifier";
    private static final String deleteString = "Supprimer";
    private static final String upString = "Monter";
    private static final String downString = "Descendre";
    private JButton addButton;
    private JButton modifyButton;
    private JButton deleteButton;
    private JButton upButton;
    private JButton downButton;
    private JTextField editField;
    
    /**
       Mthode chargeant en mmoire le fichier de mots vides.
    */
    public void loadFromEmptyFile()
    {
	String anEmptyWord = "";
	try
	    {
		BufferedReader in = new BufferedReader(new FileReader(System.getProperty("user.dir") + System.getProperty("file.separator") + DATAS_PATH + System.getProperty("file.separator") + EMPTY_WORD_FILE));
		
		while(in.ready())
		    {
			anEmptyWord = in.readLine();
			listModel.addElement(anEmptyWord);
		    }
		in.close();
	    }
	catch (IOException e)
	    {
		System.err.println(e.getLocalizedMessage());
		System.exit(1);
	    }
    }
    
    /**
       Mthode sauvegardant la liste des mots vides dans un fichier.
    */
    public void saveToEmptyFile()
    {
	PrintWriter outToEmptyFile = null;
	try
	    {
		outToEmptyFile = new PrintWriter(new BufferedWriter(new FileWriter(System.getProperty("user.dir") + System.getProperty("file.separator") + DATAS_PATH + System.getProperty("file.separator") + EMPTY_WORD_FILE)));
	    }
	catch (IOException e)
	    {
		System.err.println("Erreur d'entre/sortie sur le fichier de sortie : " + System.getProperty("user.dir") + System.getProperty("file.separator") + DATAS_PATH + System.getProperty("file.separator") + EMPTY_WORD_FILE);
		System.err.println("    " +  e.getLocalizedMessage());
	    }
	for (int ind = 0; ind < listModel.getSize(); ind++)
	    {
		outToEmptyFile.println(listModel.getElementAt(ind).toString());
	    }
	outToEmptyFile.flush();
	outToEmptyFile.close();
    }

    /**
       Dfini un panel servant  l'dition des mots vides.
    */
    public EmptyWordsPanel()
    {
	listModel = new DefaultListModel();
	loadFromEmptyFile();
        list = new JList(listModel);
        list.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        list.setSelectedIndex(0);
        list.addListSelectionListener(this);
        JScrollPane listScrollPane = new JScrollPane(list);

        addButton = new JButton(addString);
        addButton.setActionCommand(addString);
        addButton.addActionListener(new AddButtonListener());

	modifyButton = new JButton(modifyString);
	modifyButton.setActionCommand(modifyString);
        modifyButton.addActionListener(new ModifyButtonListener());

        deleteButton = new JButton(deleteString);
        deleteButton.setActionCommand(deleteString);
        deleteButton.addActionListener(new DeleteButtonListener());

        upButton = new JButton(new ImageIcon("images/up.gif"));
        upButton.setMargin(new Insets(0,0,0,0));
        upButton.setActionCommand(upString);
        upButton.addActionListener(new UpDownListener());

        downButton = new JButton(new ImageIcon("images/down.gif"));
        downButton.setMargin(new Insets(0,0,0,0));
        downButton.setActionCommand(downString);
        downButton.addActionListener(new UpDownListener());

        JPanel upDownPanel = new JPanel(new GridLayout(2, 1));
        upDownPanel.add(upButton);
        upDownPanel.add(downButton);

        editField = new JTextField(10);
        editField.addActionListener(new AddButtonListener());
        String name = listModel.getElementAt(list.getSelectedIndex())
                               .toString();
        editField.setText(name);

        JPanel buttonPane = new JPanel();
        buttonPane.add(editField);
        buttonPane.add(addButton);
	buttonPane.add(modifyButton);
        buttonPane.add(deleteButton);
        buttonPane.add(upDownPanel);

	setLayout(new BorderLayout());
	add(buttonPane, BorderLayout.NORTH);
	add(listScrollPane, BorderLayout.CENTER);
    }

    /**
       Classe interne grant les clicks sur le bouton "Supprimer".
    */
    class DeleteButtonListener implements ActionListener 
    {
	/**
	   Mthode invoque lors d'un click sur le bouton "supprimer".
	*/
        public void actionPerformed(ActionEvent e)
	{
            ListSelectionModel lsm = list.getSelectionModel();
            int firstSelected = lsm.getMinSelectionIndex();
            int lastSelected = lsm.getMaxSelectionIndex();
            listModel.removeRange(firstSelected, lastSelected);
	    
            int size = listModel.size();
	    
            if (size == 0) 
		{
		    // la liste est vide : nous dsactivons les boutons "supprimer" et de dplacement.
                deleteButton.setEnabled(false);
                upButton.setEnabled(false);
                downButton.setEnabled(false);

		} 
	    else 
		{
		    if (firstSelected == listModel.getSize())
			{
			    firstSelected--;
			}
		    list.setSelectedIndex(firstSelected);
            }
        }
    }

    /**
       Classe interne grant les clicks sur le bouton "Ajouter".
    */
    class AddButtonListener implements ActionListener
    {
	/**
	   Mthode invoque lors d'un click sur le bouton "Ajouter".
	*/
        public void actionPerformed(ActionEvent e) 
	{
            if (editField.getText().equals("")) 
		{
		    // l'utilisateur n'a fait aucune saisie...
		    Toolkit.getDefaultToolkit().beep();
		    return;
		}
	    
            int index = list.getSelectedIndex();
            int size = listModel.getSize();
	    
            // si aucune slection ou si en dernire position dans la liste,
            // nous ajoutons le nouvel lment en fin de liste.
            if (index == -1 || (index+1 == size)) 
		{
		    listModel.addElement(editField.getText());
		    list.setSelectedIndex(size);
		    
		    // Dans le cas contraire nous ajoutons le nouvel lment aprs l'lment slectionn.
		} 
	    else 
		{
		    listModel.insertElementAt(editField.getText(), index+1);
		    list.setSelectedIndex(index+1);
		}
        }
    }

    /**
       Classe interne grant les clicks sur le bouton "Modifier".
    */
    class ModifyButtonListener implements ActionListener 
    {
	/**
	   Mthode invoque lors d'un click sur le bouton "Modifier".
	*/
        public void actionPerformed(ActionEvent e)
	{
            if (editField.getText().equals(""))
		{
		    // la zone de saisie est vide...
		    Toolkit.getDefaultToolkit().beep();
		    return;
		}
	    
            int index = list.getSelectedIndex();
	    int size = listModel.getSize();
	    
            if (index != -1 )
		{
		    listModel.setElementAt(editField.getText(), index);
		    list.setSelectedIndex(index);
		}
        }
    }

    /**
       Classe interne grant les clicks sur les bouton de dplacement.
    */
    class UpDownListener implements ActionListener 
    {
	/**
	   Mthode invoque lors d'un click sur les boutons de dplacement.
	*/
        public void actionPerformed(ActionEvent e)
	{
	    int moveMe = list.getSelectedIndex();

            if (e.getActionCommand().equals(upString))
		{
		    // dplacement vers le haut
		    if (moveMe != 0) 
			{     
			    // pas encore au sommet
			    swap(moveMe, moveMe-1);
			    list.setSelectedIndex(moveMe-1);
			    list.ensureIndexIsVisible(moveMe-1);
			}
		}
	    else 
		{
		    // dplacement vers le bas
		    if (moveMe != listModel.getSize()-1)
			{
			    // pas encore  la base
			    swap(moveMe, moveMe+1);
			    list.setSelectedIndex(moveMe+1);
			    list.ensureIndexIsVisible(moveMe+1);
			}
		}
        }
    }
    
    // change deux lments de la liste.
    private void swap(int a, int b)
    {
        Object aObject = listModel.getElementAt(a);
        Object bObject = listModel.getElementAt(b);
        listModel.set(a, bObject);
        listModel.set(b, aObject);
    }
    
    /**
       Surcharge de la mthode de gestion des changements de valeurs dans la list.
    */
    public void valueChanged(ListSelectionEvent e) 
    {
        if (e.getValueIsAdjusting() == false)
	    {
		if (list.getSelectedIndex() == -1) 
		    {
			//Aucune slection : dsactive les boutons.
			deleteButton.setEnabled(false);
			upButton.setEnabled(false);
			downButton.setEnabled(false);
			editField.setText("");
			
		    }
		else if (list.getSelectedIndices().length > 1) 
		    {
			//slection multiple : dsactive les boutons de dplacement.
			deleteButton.setEnabled(true);
			upButton.setEnabled(false);
			downButton.setEnabled(false);
			
		    }
		else 
		    {
			// une slection simple permet toutes les oprations.
			deleteButton.setEnabled(true);
			upButton.setEnabled(true);
			downButton.setEnabled(true);
			editField.setText(list.getSelectedValue().toString());
		    }
	    }
    }
    
}
