/*
  Termin
*/

/*****************************************************
ThemeEditor 1.0 est un diteur de thmes au format XML
Copyright (C) 2000 David MASCLE et Damien BOUCHARD.
Auteurs: David MASCLE et Damien BOUCHARD.
******************************************************/

import java.util.*;
import java.lang.*;
import java.io.*;
import javax.swing.*;

/************************************************************************************************
<b>FreqJList</b> tablis la liste des mots d'un texte avec leurs frquences et l'affiche.<br><br>
<br><b>ThemeEditor 1.0</b> est un diteur de thmes au format XML.<br>
<br>Copyright &copy; 2000 David MASCLE et Damien BOUCHARD.<br>

@version : 1.0
@author : David MASCLE et Damien BOUCHARD

************************************************************************************************/

public class FreqJList extends JList
{
    /**
       Objet servant  tokeniser le ou les textes
    */
    private MyStreamTokenizer st = null;
    /**
       Objet de stockage des mots vides
    */
    private EmptyWordsMap ewMap = null;
    private String ewFileName = "";
    private static final Integer ONE = new Integer(1);
    
    /**
       Dfini une JList de frquence.
       
       @param emptyWordsFileName nom du fichier de mots vides.
                         
    */
    public FreqJList(String emptyWordsFileName)
    {
	String tabtemp[] = null;
	setFiles(tabtemp);
	ewFileName = emptyWordsFileName;
	//ewMap = new EmptyWordsMap(emptyWordsFileName);
    }
    
    /**
       Lance le calcul de frquences sur le ou les fichiers passs en paramtres.
       
       @param fileNames nom du fichier ou des fichiers pour le calcul de frsquence.
                         
    */
    public void setFiles(String[] fileNames) 
    {
	if (fileNames == null)
	    {
		// cas ou aucun fichier n'est pass
		String[] data = {"                                        "};
		setListData(data);
	    }
	else
	    {
		// initialisation de la table de hachage de mots vides
		ewMap = new EmptyWordsMap(ewFileName);
		// construction d'une numrcd deation de flots sur les fichiers pour le calcul de frquence.
		ListOfFiles mylist = new ListOfFiles(fileNames);
		// ouverture d'un lecteur sur le flot de fichiers
		Reader r = new BufferedReader(new InputStreamReader(new SequenceInputStream(mylist)));
		// table de hachage servant  stocker les mots avec leurs frquences
		Map m = new TreeMap();
		// tokenisation du ou des fichiers
		String token = null;
		try
		    {		
			st = new MyStreamTokenizer(r);
			
			while (st.nextToken() != st.TT_EOF)
			    {
				if (st.ttype == st.TT_WORD)
				    {
					token = st.sval;
					if (!ewMap.containsWord(token))
					    {
						Integer freq = (Integer) m.get(token);
						m.put(token, (freq==null ? ONE : new Integer(freq.intValue() + 1)));
					    }
				    }
				else 
				    if (st.ttype == st.TT_NUMBER)
					{
					    token = (new Double(st.nval)).toString();
					    if (!ewMap.containsWord(token))
						{
					    Integer freq = (Integer) m.get(token);
					    m.put(token, (freq==null ? ONE : new Integer(freq.intValue() + 1)));
						}
					}
			    }
		    }
		catch (Exception e) 
		    {
			System.err.println("Erreur de tokenization du ou des  fichiers : ");
			for (int indf = 0; indf < fileNames.length; indf++)
			    System.err.println(" ==> " + fileNames[indf]);
			System.err.println("    " +  e.getLocalizedMessage());
		    }
		
		st = null;
		
		try
		    {
			// fermeture du lecteur
			r.close();
			r = null;
		    }
		catch (IOException e)
		    {
			System.err.println("Erreur d'entre/sortie sur le ou les fichiers : ");
			for (int indf = 0; indf < fileNames.length; indf++)
			    System.err.println(" ==> " + fileNames[indf]);
			System.err.println("    " +  e.getLocalizedMessage());
		    }
		
		// nous crons un tableau de tous les mots avec leurs frquences
		WordAndFrequency tableau[] = null;
		tableau = new WordAndFrequency[m.size()];
		Iterator iter = m.keySet().iterator();
		
		int cpt = 0;
		
		while (iter.hasNext())
		    {
			String key = iter.next().toString();
			tableau[cpt++] = new WordAndFrequency(key, ((Integer)m.get(key)).intValue());
		    }
		
		// nous trions le tableau de mots par ordre dcroissant de frquence
		quickSort(tableau);
		// nous affichons le tableau dans la JList
		this.setListData(tableau);
	    }
    }

    private WordAndFrequency[] quickSort(WordAndFrequency data[],int left,int right) 
    { 
	// mthode de tri du tableau de mots avec leurs frquences
	WordAndFrequency tmp; 
	int original_left = left; 
	int original_right = right; 
	int mid = data[(left + right) / 2].getFrequency();
	do 
	    { 
		while(data[left].getFrequency() > mid) 
		    { 
			left++; 
		    } 
		while (mid > data[right].getFrequency())
		    { 
			right--; 
		    } 
		if (left <= right)
		    { 
			tmp = data[left]; 
			data[left] = data[right]; 
			data[right] = tmp; 
			left++; 
			right--; 
		    } 
	    } while (left <= right); 
	if (original_left < right) data=quickSort(data, original_left, right); 
	if (left < original_right) data=quickSort(data, left, original_right); 
	return data;		
    } 
    
    
    private WordAndFrequency[] quickSort(WordAndFrequency a[]) 
    {
	// mthode de tri du tableau de mots avec leurs frquences
	if (a.length <= 1) return a;
	return quickSort(a, 0, a.length-1);
    }
}
