/*
  Termin
*/

/******************************************************
ThemeEditor 1.0 est un diteur de thmes au format XML.
Copyright (C) 2000 David MASCLE et Damien BOUCHARD.
Auteurs: David MASCLE et Damien BOUCHARD.
*******************************************************/

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import java.awt.*;
import java.net.URL;
import java.net.MalformedURLException;
import java.io.IOException;

/*****************************************************************************
<b>HtmlPanel</b> est un visualiseur Html.<br><br>
<br><b>ThemeEditor 1.0</b> est un diteur de thmes au format XML.<br>
<br>Copyright &copy; 2000 David MASCLE et Damien BOUCHARD.<br>

@version : 1.0
@author : David MASCLE et Damien BOUCHARD

******************************************************************************/

public class HtmlPanel extends JPanel implements HyperlinkListener 
{
    private JEditorPane html =  null;
    private Cursor defaultCursor = null;
    private Cursor handCursor = null;

    /**
       Dfini un panel contenant un visualiseur Html.

       @param urlStr une url  afficher dans le visualiseur Html.
    */
    public HtmlPanel(String urlStr)
    {
	defaultCursor = new Cursor(Cursor.DEFAULT_CURSOR);
	handCursor = new Cursor(Cursor.HAND_CURSOR);
	setLayout(new BorderLayout());
	try 
	    {
		URL url = null;
		try 
		    {
			url = new URL(urlStr);
		    } 
		catch (java.net.MalformedURLException exc)
		    {
			System.err.println("L'url  " + url + " n'est pas correctement form !");
			url = null;
		    }
		
		if(url != null) 
		    {
			html = new JEditorPane(url);
			html.setEditable(false);
			html.addHyperlinkListener(this);
			JScrollPane scroller = new JScrollPane();
			JViewport vp = scroller.getViewport();
			vp.add(html);
			vp.setBackingStoreEnabled(true);
			add(scroller, BorderLayout.CENTER);
		    }
	    } 
	
	catch (IOException e) 
	    {
		System.err.println("Erreur d'entre/Sortie : " + e.getLocalizedMessage());
	    }
    }

    /**
       Cette mthode permet de changer la page web affiche dans le visualiseur Html.
       
       @param urlStr une url  afficher dans le visualiseur Html.
    */
    public void setPage(String urlStr)
    {
	URL url = null;
	try 
	    {
		url = new URL(urlStr);
	    } 
	catch (java.net.MalformedURLException exc)
	    {
		System.err.println("L'url  " + url + " n'est pas correctement form !");
		url = null;
	    }
	
	if(url != null) 
	    {
		linkActivated(url);
	    }
    } 
    
    
    /**
       Notification d'un changement concernant un lien hypertexte.

       @param e un objet indiquant le type d'vnement survenant sur le lien hypertexte.
    */
    public void hyperlinkUpdate(HyperlinkEvent e)
    {
        if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
	    {
		linkActivated(e.getURL());
	    }
	else if (e.getEventType() == HyperlinkEvent.EventType.ENTERED)
	    {
		html.setCursor(handCursor);
	    }
	else if (e.getEventType() == HyperlinkEvent.EventType.EXITED)
	    {
		html.setCursor(defaultCursor);
	    }
    }

    /**
       Mthode appele lors de l'activation d'un lien dans le visualiseur html.

       @param u l'URL  suivre.
    */
    protected void linkActivated(URL u)
    {
        Cursor c = html.getCursor();
        Cursor waitCursor = Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);
        html.setCursor(waitCursor);
        SwingUtilities.invokeLater(new PageLoader(u, c));
    }

    /**
       classe temporaire permettant le chargement d'une page Web de faon synchrone.
    */
    class PageLoader implements Runnable
    {
        
        PageLoader(URL u, Cursor c)
	{
            url = u;
            cursor = c;
        }
	
        public void run()
	{
            if (url == null) 
		{
		    // restaure le prcdent curseur
		    html.setCursor(cursor);
		    Container parent = html.getParent();
		    parent.repaint();
		}
	    else 
		{
		    Document doc = html.getDocument();
		    try
			{
			    // chargement de la page
			    html.setPage(url);
			}
		    catch (IOException ioe)
			{
			    html.setDocument(doc);
			    getToolkit().beep();
			}
		    finally 
			{
			    url = null;
			    SwingUtilities.invokeLater(this);
			}
		}
        }
	
        URL url;
        Cursor cursor;
    }
}
