/*****************************************************
ThemeEditor 1.0 est un diteur de thmes au format XML
Copyright (C) 2000 David MASCLE et Damien BOUCHARD.
Auteurs: David MASCLE et Damien BOUCHARD.
******************************************************/

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import javax.swing.border.*;

/***********************************************************************
<b>TheFileChooser</b> affiche la liste des textes et permet le lancement
du calcul des occurrences et du coloriage.<br><br>
<br><b>ThemeEditor 1.0</b> est un diteur de thmes au format XML.<br>
<br>Copyright &copy; 2000 David MASCLE et Damien BOUCHARD.<br>
 
@version : 1.0
@author : David MASCLE et Damien BOUCHARD
************************************************************************/

public class TheFileChooser extends JPanel
{
    private GridBagConstraints gridbagC = null;
    private GridBagLayout gridbagL = null;
    private FreqJListPanel freqJListPanel = null;
    private HtmlPanel htmlPanel = null;
    private JList fileList = null;
    private JButton colorButton = null;
    private JButton occurrencesButton = null;
    private ThemeEditorVars teVars = null;

    /**
        Dfini un afficheur de la liste des textes sur lesquels nous pouvons lancer un calcul d'occurrences ou un coloriage.
	@param teVars les variables partages de ThemeEditor.
	@param freq la classe de calcul et d'affichage des occurrences.
	@param html le visualiseur html.
     */
    public TheFileChooser(ThemeEditorVars teVars, FreqJListPanel freq, HtmlPanel html)
    {
	this.teVars = teVars;
	freqJListPanel = freq;
	htmlPanel = html;
	gridbagL = new GridBagLayout();
	setLayout(gridbagL);
	gridbagC = new GridBagConstraints();
	gridbagC.gridx = 0;
	gridbagC.gridy = 0;
	gridbagC.gridwidth = 2;
	gridbagC.gridheight = 1;
	gridbagC.weightx = 1.0;
	gridbagC.weighty = 1.0;
	gridbagC.fill = GridBagConstraints.BOTH;
	
	// la liste des textes
	fileList = new JList();
	JScrollPane fileListScrollPane = new JScrollPane(fileList);
	//JFileChooser fc = new JFileChooser("texts");
	
	
	//FileSystemView fsv = fc.getFileSystemView();
	//File path = new File("texts");
	//System.err.println("Rpertoire : " + path);

	//File [] files = fsv.getFiles(new File("./texts"),false);

	File repertoire = new File("./texts");
	File[] files    = repertoire.listFiles();

	fileList.setListData(files);
	add(fileListScrollPane, gridbagC);

	// le bouton de lancement du coloriage
	colorButton = new JButton("Colorier", new ImageIcon("images/color.gif"));
	colorButton.setToolTipText("Colorie le ou les textes slectionns ci-dessus");
	colorButton.setDoubleBuffered(true);
	colorButton.addActionListener(new ActionListener() 
	    {
		public void actionPerformed(ActionEvent e) 
		{
		    colorButtonPressed();
	       }
	    });
	gridbagC = new GridBagConstraints();
	gridbagC.gridx = 0;
	gridbagC.gridy = 1;
	gridbagC.fill = GridBagConstraints.BOTH;
	add(colorButton, gridbagC);
	
	// le bouton de calcul des occurrences
	occurrencesButton = new JButton("Calculer les occurrences", new ImageIcon("images/occurrences.gif"));
	occurrencesButton.setToolTipText("Calcule les occurrences du ou des textes slectionns ci-dessus");
	occurrencesButton.setDoubleBuffered(true);
	occurrencesButton.addActionListener(new ActionListener() 
	    {
		public void actionPerformed(ActionEvent e) 
		{
		    occurrencesButtonPressed();
	       }
	    });
	gridbagC = new GridBagConstraints();
	gridbagC.gridx = 1;
	gridbagC.gridy = 1;
	gridbagC.fill = GridBagConstraints.BOTH;
	add(occurrencesButton, gridbagC);

	setVisible(true);
    }

    /**
       Mthode invoque pour le coloriage d'un ou plusieurs textes slectionns.
    */
    public void colorButtonPressed() 
    {
	if (teVars.themeFileNameIsDefined())
	    {
		boolean stillContinue = true;
		
		if (teVars.themeFileHasBeenModified())
		    {
			if (JOptionPane.showConfirmDialog(this, "Le fichier de thmes en cours d'dition a t modifi,\n souhaitez vous quand mme effectuer le coloriage ?", "Confirmation du coloriage", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) == JOptionPane.NO_OPTION)
			    {
				stillContinue = false;
			    }
		    }
		
		if (stillContinue)
		    {
			fileList.setEnabled(false);
			colorButton.setEnabled(false);
			occurrencesButton.setEnabled(false);
			
			htmlPanel.setPage("file:"  + System.getProperty("user.dir") + System.getProperty("file.separator") + "applicationdatas" + System.getProperty("file.separator") + "blank.html");
			Object[] objListTemp = fileList.getSelectedValues();
			String[] listOfFiles = new String[objListTemp.length];
			for (int ind = 0; ind < objListTemp.length; ind++)
			    listOfFiles[ind] = objListTemp[ind].toString();
			
			ColoringClass coloringClass = new ColoringClass(System.getProperty("user.dir") + System.getProperty("file.separator") + "applicationdatas" + System.getProperty("file.separator") + "empty.txt", teVars.getThemeFileName(), listOfFiles, "");
			
			coloringClass.color();
			
			htmlPanel.setPage("file:"  + System.getProperty("user.dir") + System.getProperty("file.separator") + "output" + System.getProperty("file.separator") + "output.html");
			
			fileList.setEnabled(true);
			colorButton.setEnabled(true);
			occurrencesButton.setEnabled(true);
		    }
	    }
	else
	    {
		JOptionPane.showMessageDialog(this, "Aucun fichier de thmes n'a t slectionn, pour cela,\n vous devez soit ouvrir un fichier de thmes,\n soit enregistrer la liste de thmes en cours d'dition.", "Spcification du fichier de thmes", JOptionPane.ERROR_MESSAGE);
	    }
    }
    
    /**
       Mthode invoque pour le calcul d'occurrences sur un ou plusieurs textes slectionns.
    */
    public void occurrencesButtonPressed() 
    {
	fileList.setEnabled(false);
	colorButton.setEnabled(false);
	occurrencesButton.setEnabled(false);
	
	// nous rcuprons la liste des fichiers
	Object[] objListTemp = fileList.getSelectedValues();
	String[] listOfFiles = new String[objListTemp.length];
	
	// nous rcuprons le nom de chaque fichier
	for (int ind = 0; ind < objListTemp.length; ind++)
	    listOfFiles[ind] = objListTemp[ind].toString();
	
	// nous lancons le calcul d'occurrences
	freqJListPanel.setFiles(listOfFiles);
	
	fileList.setEnabled(true);
	colorButton.setEnabled(true);
	occurrencesButton.setEnabled(true);
    }
}
