/*
  Termin
*/

/******************************************************
ThemeEditor 1.0 est un diteur de thmes au format XML.
Copyright (C) 2000 David MASCLE et Damien BOUCHARD.
Auteurs: David MASCLE et Damien BOUCHARD.
*******************************************************/

import java.util.*;

/************************************************************************************************
<b>Theme</b> est une classe conteneur pour un thme utilise pour le coloriage de textes.<br><br>
<br><b>ThemeEditor 1.0</b> est un diteur de thmes au format XML.<br>
<br>Copyright &copy; 2000 David MASCLE et Damien BOUCHARD.<br>

@version : 1.0
@author : David MASCLE et Damien BOUCHARD

*************************************************************************************************/

public class Theme implements Comparable
{
    private String definition = ""; // identifiant du thme
    private String color = ""; // couleur associe au thme
    private Map wordsOfTheTheme = null; // les mots contenus dans le thme
    private int numberOfWordsColored = 0; // nombre de mots coloris provenant de ce thme

    /**
       Dfini un nouveau thme.
       
       @param definition la dfinition du thme.
    */
    public Theme(String definition)
    {
	this.definition = definition;
	// nous choisissons une couleur au hasard
	int redTemp = new Double(Math.random() * 255).intValue();
	if (redTemp < 20)
	    redTemp = redTemp + new Double(Math.random() * 20).intValue();
	int greenTemp = new Double(Math.random() * 255).intValue();
	if (greenTemp < 30)
	    greenTemp = greenTemp + new Double(Math.random() * 30).intValue();
	int blueTemp = new Double(Math.random() * 255).intValue();
	if (blueTemp < 40)
	    blueTemp = blueTemp + new Double(Math.random() * 40).intValue();
	String redComponent = Integer.toHexString(redTemp);
	String greenComponent = Integer.toHexString(greenTemp);
	String blueComponent = Integer.toHexString(blueTemp);
	color = "#" + redComponent + greenComponent + blueComponent;
	wordsOfTheTheme = new HashMap();
	numberOfWordsColored = 0;
    }

    /**
       Redfinition de la mthode de comparaison pour la classe Thme.

       @param o un object qui doit etre de type Theme.
     */
    public int compareTo(Object o)
    {
	Theme t = (Theme) o;
	return (new Integer(numberOfWordsColored)).compareTo(new Integer(t.numberOfWordsColored));
    }

    /**
       Mthode servant  ajouter un mot au thme.

       @param word le mot  ajouter.
     */
    public void addWord(String word)
    {
	wordsOfTheTheme.put(word.toLowerCase(), new Integer(0));
    }
    
    /**
       Mthode servant  redfinir la dfinitiondu thme.

       @param definition la nouvelle dfinition pour le thme.
     */
    public void setDefinition(String definition)
    {
	this.definition = definition;
    }

    /**
       Mthode retournant la dfinition associe au thme.
       
       @return la dfinition associe au thme.
     */
    public String getDefinition()
    {
	return definition;
    }

    /**
       Mthode retournant la couleur associe au thme.
       
       @return la couleur associe au thme.
     */
    public String getColor()
    {
	return color;
    }

    /**
       Mthode retournant le nombre de mots appartenant au thme.

       @returns le nombre de mots appartenant au thme.
     */
    public int getNumberOfWordsInTheTheme()
    {
	return wordsOfTheTheme.size();
    }
    
    /**
       Mthode retournant le nombre de mots coloris avec ce thme.

       @return le nombre de mots coloris avec ce thme.
     */
    public int getNumberOfWordsColored()
    {
	return numberOfWordsColored;
    }

    /**
       Mthode retournant le nombre de mots diffrents du thme utiliss pour colorier.

       @return le nombre de mots diffrents du thme utiliss pour colorier.
     */
    public int getNumberOfWordsUsedToColor()
    {
	int result = 0;
	Iterator iter = wordsOfTheTheme.keySet().iterator();
	while (iter.hasNext())
	    {
		String key = iter.next().toString();
		if (((Integer)wordsOfTheTheme.get(key)).intValue() > 0)
		    result++;
	    }
	return result;
    }

    /**
       Mthode incrmentant le nombre de mots coloris avec le thme.
     */
    public void incrementNumberOfWordsColored()
    {
	numberOfWordsColored++;
    }
    
    /**
       Mthode servant  incrmenter le nombre d'utilisations du mot pass en paramtre lors du coloriage.

       @param word le mot.
     */
    public void incrementNumberOfWordsColoredWith(String word)
    {
	Integer nb = (Integer)wordsOfTheTheme.get(word.toLowerCase());
	wordsOfTheTheme.put(word.toLowerCase(), new Integer(nb.intValue() + 1));
    }

    /**
       Mthode retournant true si le mot pass en paramtre appartient au thme et false dans le cas contraire.

       @param word le mot dont nous voulons vrifier l'appartenance au thme.

       @return true si le mot appartient au thme et false sinon.
     */
    public boolean containsWord(String word)
    {
	return wordsOfTheTheme.containsKey(word.toLowerCase());
    }
}
