/*
  Termin
*/

/*****************************************************
ThemeEditor 1.0 est un diteur de thmes au format XML
Copyright (C) 2000 David MASCLE et Damien BOUCHARD.
Auteurs: David MASCLE et Damien BOUCHARD.
******************************************************/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.util.*;

/*********************************************************************
<b>ThemeEditor</b> est la fenetre principale de l'application.
<br><b>ThemeEditor 1.0</b> est un diteur de thmes au format XML.<br>
<br>Copyright &copy; 2000 David MASCLE et Damien BOUCHARD.<br>
 
@version : 1.0
@author : David MASCLE et Damien BOUCHARD
**********************************************************************/

public class ThemeEditor extends JFrame implements ThemeEditorConstants
{
    private NewAction newAction = null;
    private OpenAction openAction = null;
    private SaveAction saveAction = null;
    private SaveAsAction saveAsAction = null;
    private CloseAction closeAction = null;
    private QuitAction quitAction = null;
    private EmptyWordsAction emptyWordsAction = null;
    private ConfigureOptionsAction configureOptionsAction = null;
    private HelpIndexAction helpIndexAction = null;
    private AboutAction aboutAction = null;
    private ThemeEditorVars teVars = null;
    private ThemeTree themeTree = null;
    private Properties prop = null;

    /**
       Mthode principale lanant l'application
       @param args[] la ligne de commande
    */
    public static void main(String args[])
    {
	String fileParam = "";
	
	if (args.length >= 1)
	    fileParam = args[0];
	
	new ThemeEditor("ThemeEditor 1.0 - Copyright  2000 David MASCLE et Damien BOUCHARD", fileParam);
    }
    
    /**
       Charge le fichier d'initialisation de l'application.
    */
    private void loadIniFile()
    {
	String iniFileName = System.getProperty("user.dir") + System.getProperty("file.separator") + DATAS_PATH + System.getProperty("file.separator") + INI_FILE;
	FileInputStream fIn = null;
	try
	    {
		fIn = new FileInputStream(iniFileName);
	    }
	catch (FileNotFoundException e)
	    {
		System.err.println(e.getLocalizedMessage());
	    }
	
	try
	    {
		prop.load(fIn);
		fIn.close();
	    }
	catch (IOException e)
	    {
		System.err.println(e.getLocalizedMessage());
	    }
    }

    /**
       Enregistre les paramtres de l'application dans le fichier d'initialisation de l'application.
    */
    private void saveIniFile()
    {
	prop.setProperty("UseDatabase", (new Boolean(teVars.getUseDatabase())).toString());
	prop.setProperty("MysqlServerAddress", teVars.getMysqlServerAddress());
	prop.setProperty("MysqlServerPort", teVars.getMysqlServerPort());
	prop.setProperty("DatabaseName", teVars.getDatabaseName());
	prop.setProperty("UserName", teVars.getUserName());
	prop.setProperty("UserPassword", teVars.getUserPassword());
	
	prop.setProperty("xPos", (new Integer((int)getLocation().getX())).toString());
	prop.setProperty("yPos", (new Integer((int)getLocation().getY())).toString());
	prop.setProperty("fWidth", (new Integer((int)getSize().getWidth())).toString());
	prop.setProperty("fHeight",(new Integer((int) getSize().getHeight())).toString());

	PrintWriter outToIniFile = null;
	String iniFileName = System.getProperty("user.dir") + System.getProperty("file.separator") + DATAS_PATH + System.getProperty("file.separator") + INI_FILE;
	try
	    {
		outToIniFile = new PrintWriter(new BufferedWriter(new FileWriter(iniFileName)));
	    }
	catch (IOException e)
	    {
		System.err.println("Erreur d'entre/sortie sur le fichier de sortie : " + iniFileName);
		System.err.println("    " +  e.getLocalizedMessage());
	    }
	prop.list(outToIniFile);
	outToIniFile.flush();
	outToIniFile.close();
    }
    
    /**
       Dfini la fenetre de l'application ThemeEditor.
       @param title le titre de la fenetre.
       @param fileName le nom d'un ventuel fichier de thmes.
    */
    public ThemeEditor(String title,String fileName)
    {
	super(title);
	prop = new Properties();
	// chargement du fichier d'initialisation
	loadIniFile();
	setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
	// paramtrage des variables de ThemeEitor
	teVars = new ThemeEditorVars(fileName, false, (new Boolean(prop.getProperty("UseDatabase", "false"))).booleanValue(), prop.getProperty("MysqlServerAddress", ""), prop.getProperty("MysqlServerPort", ""), prop.getProperty("DatabaseName", ""), prop.getProperty("UserName", ""), prop.getProperty("UserPassword", ""));

	// la zone d'dition des thmes
	themeTree = new ThemeTree(teVars);
	// si un fichier de thmes a t pass en paramtre, nous le chargeons en mmoire
	if (teVars.themeFileNameIsDefined())
	    themeTree.load();

	newAction = new NewAction(teVars, themeTree);
	openAction = new OpenAction(teVars, themeTree);
	saveAction = new SaveAction(teVars, themeTree);
	saveAsAction = new SaveAsAction(teVars, themeTree);
	closeAction = new CloseAction(teVars, themeTree);
	quitAction = new QuitAction(teVars);
	emptyWordsAction = new EmptyWordsAction(this);
	configureOptionsAction = new ConfigureOptionsAction(this, teVars);
	helpIndexAction = new HelpIndexAction(this);
	aboutAction = new AboutAction(this);
	// la barre de menu
	JMenuBar menuBar = new ThemeEditorMenuBar(newAction, openAction, saveAction, saveAsAction, closeAction, quitAction, emptyWordsAction, configureOptionsAction, helpIndexAction, aboutAction);
	// la barre de bouton
	ThemeEditorToolBar toolBar = new ThemeEditorToolBar(newAction, openAction, saveAction, emptyWordsAction, configureOptionsAction, helpIndexAction);
	getContentPane().setLayout(new BorderLayout());
	setJMenuBar(menuBar);
	// la liste des occurrences
	FreqJListPanel freqJListPanel = new FreqJListPanel("applicationdatas/empty.txt", themeTree);
	// le visualiseur HTML
	HtmlPanel htmlPanel = new HtmlPanel("file:" + System.getProperty("user.dir") + System.getProperty("file.separator") + "applicationdatas" + System.getProperty("file.separator") + "blank.html");
	// le slecteur de fichiers
	TheFileChooser theFileChooser = new TheFileChooser(teVars, freqJListPanel, htmlPanel);
	// placement des diffrents composants de l'application
	JSplitPane topRightSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, theFileChooser, freqJListPanel);
	topRightSplitPane.setOneTouchExpandable(true);
	JSplitPane topSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, themeTree, topRightSplitPane);
	topSplitPane.setOneTouchExpandable(true);
	JSplitPane mainSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, topSplitPane, htmlPanel);
	mainSplitPane.setOneTouchExpandable(true);
	getContentPane().add(toolBar, BorderLayout.NORTH);
	getContentPane().add(mainSplitPane, BorderLayout.CENTER);
	
	addWindowListener(new WindowClosing(this, teVars));		

	// positionnement et dimensionnement de la fenetre
	int xPos = 0;
	int yPos = 0;
	int fWidth = 640;
	int fHeight = 480;
	xPos = Integer.parseInt(prop.getProperty("xPos", "0"));
	yPos = Integer.parseInt(prop.getProperty("yPos", "0"));
	fWidth = Integer.parseInt(prop.getProperty("fWidth", "640"));
	fHeight = Integer.parseInt(prop.getProperty("fHeight", "480"));
        setLocation(xPos, yPos);
	setSize(new Dimension(fWidth, fHeight));
	show();	
    }


    /**
       Action "Nouveau"
    */
    private class NewAction implements ActionListener
    {
	private ThemeEditorVars teVars = null;
	private ThemeTree themeTree = null;

	public NewAction(ThemeEditorVars tev, ThemeTree tt)
	{
	    teVars = tev;
	    themeTree = tt;
	}
	/**
	   Mthode invoque lorsque l'action correspondante est active
	*/
	public void actionPerformed(ActionEvent e)
	{
	    if (!teVars.themeFileHasBeenModified())
		{
		    themeTree.clear();
		    teVars.setThemeFileName("");
		    teVars.setThemeFileIsModified(false);
		}
	    else
		{
		    if (JOptionPane.showConfirmDialog(null, "Le fichier de thmes en cours d'dition n'a pas\n t sauvegard, souhaitez vous quand mme le supprimer ?", "Cration d'un nouveau fichier de thmes", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) == JOptionPane.YES_OPTION)
			{
			    themeTree.clear();
			    teVars.setThemeFileName("");
			    teVars.setThemeFileIsModified(false);
			}
		}
	}
    }

    /**
       Action "Ouvrir"
    */
    private class OpenAction implements ActionListener
    {
	private ThemeEditorVars teVars = null;
	private ThemeTree themeTree = null;
	
	public OpenAction(ThemeEditorVars tev, ThemeTree tt)
	{
	    teVars = tev;
	    themeTree = tt;
	}
	/**
	   Mthode invoque lorsque l'action correspondante est active
	*/
	public void actionPerformed(ActionEvent e)
	{
	    if (!teVars.themeFileHasBeenModified())
		{
		    // la liste de thmes en cours d'dition n'a pas t modifie
		    JFileChooser fc = new JFileChooser(System.getProperty("user.dir") + System.getProperty("file.separator") + "themes");
		    int returnVal = fc.showOpenDialog(ThemeEditor.this);
		    if (returnVal == JFileChooser.APPROVE_OPTION) 
			{
			    File file = fc.getSelectedFile();
			    teVars.setThemeFileName(file.getPath());
			    teVars.setThemeFileIsModified(false);
			    themeTree.load();
			}
		}
	    else
		{
		    // la liste de thmes en cours d'dition a t modifie
		    if (JOptionPane.showConfirmDialog(null, "Le fichier de thmes en cours d'dition n'a pas\n t sauvegard, souhaitez vous quand mme le supprimer ?", "Ouverture d'un fichier de thmes", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) == JOptionPane.YES_OPTION)
			{
			   JFileChooser fc = new JFileChooser(System.getProperty("user.dir") + System.getProperty("file.separator") + "themes");
			   int returnVal = fc.showOpenDialog(ThemeEditor.this);
			   if (returnVal == JFileChooser.APPROVE_OPTION) 
			       {
				   File file = fc.getSelectedFile();
				   teVars.setThemeFileName(file.getPath());
				   teVars.setThemeFileIsModified(false);
				   themeTree.load();
			       }
			}
		}
	}
    }
    
    /**
       Action "Enregistrer"
    */
    private class SaveAction implements ActionListener
    {
	private ThemeEditorVars teVars = null;
	private ThemeTree themeTree = null;
	
	public SaveAction(ThemeEditorVars tev, ThemeTree tt)
	{
	    teVars = tev;
	    themeTree = tt;
	}
	/**
	   Mthode invoque lorsque l'action correspondante est active
	*/
	public void actionPerformed(ActionEvent e)
	{
	    if (teVars.themeFileNameIsDefined())
		{
		    teVars.setThemeFileIsModified(false);
		    themeTree.save();
		}
	    else
		{
		    JFileChooser fc = new JFileChooser(System.getProperty("user.dir") + System.getProperty("file.separator") + "themes");
		    int returnVal = fc.showSaveDialog(ThemeEditor.this);
		    if (returnVal == JFileChooser.APPROVE_OPTION) 
			{
			    File file = fc.getSelectedFile();
			    teVars.setThemeFileName(file.getPath());
			    teVars.setThemeFileIsModified(false);
			    themeTree.save();
			}
		}
	}
    }

    /**
       Action "Enregistrer sous"
    */
    private class SaveAsAction implements ActionListener
    {
	private ThemeEditorVars teVars = null;
	private ThemeTree themeTree = null;

	public SaveAsAction(ThemeEditorVars tev, ThemeTree tt)
	{
	    teVars = tev;
	    themeTree = tt;
	}
	/**
	 * Mthode invoque lorsque l'action correspondante est active
	 */
	public void actionPerformed(ActionEvent e)
	{
	    JFileChooser fc = new JFileChooser(System.getProperty("user.dir") + System.getProperty("file.separator") + "themes");
	    int returnVal = fc.showSaveDialog(ThemeEditor.this);
	    if (returnVal == JFileChooser.APPROVE_OPTION) 
		{
		    File file = fc.getSelectedFile();
		    teVars.setThemeFileName(file.getPath());
		    teVars.setThemeFileIsModified(false);
		    themeTree.save();
		}
	}
    }

    /**
       Action "Fermer"
    */
    private class CloseAction implements ActionListener
    {
	private ThemeEditorVars teVars = null;
	private ThemeTree themeTree = null;

	public CloseAction(ThemeEditorVars tev, ThemeTree tt)
	{
	    teVars = tev;
	    themeTree = tt;
	}
	/**
	 * Mthode invoque lorsque l'action correspondante est active
	 */
	public void actionPerformed(ActionEvent e)
	{
	    if (teVars.themeFileNameIsDefined())
		{
		    if (!teVars.themeFileHasBeenModified())
			{
			    themeTree.clear();
			    teVars.setThemeFileName("");
			    teVars.setThemeFileIsModified(false);
			}
		    else
			{
			    if (JOptionPane.showConfirmDialog(null, "Le fichier de thmes en cours d'dition n'a pas\n t sauvegard, souhaitez vous quand mme le supprimer ?", "Fermeture du fichier de thmes", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) == JOptionPane.YES_OPTION)
				{
				    themeTree.clear();
				    teVars.setThemeFileName("");
				    teVars.setThemeFileIsModified(false);
				}
			}
		}
	}
    }
    
    /**
       Action "Quitter"
    */
    private class QuitAction implements ActionListener
    {
	private ThemeEditorVars teVars = null;
	
	public QuitAction(ThemeEditorVars tev)
	{
	    teVars = tev;
	}
	/**
	   Mthode invoque lorsque l'action correspondante est active
	*/
	public void actionPerformed(ActionEvent e)
	{
	    if (teVars.themeFileHasBeenModified())
		{
		    if (JOptionPane.showConfirmDialog(null, "Le fichier de thmes en cours d'dition n'a pas t sauvegard,\n souhaitez vous quand mme quitter l'application ?", "Quitter ThemeEditor", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) == JOptionPane.YES_OPTION)
			{
			    saveIniFile();
			    System.exit(0);
			}
		}
	    else 
		{
		    saveIniFile();
		    System.exit(0);
		}
	}
    }
    
    /**
       Action "Mots vides"
    */
    private class EmptyWordsAction implements ActionListener
    {
	ThemeEditor mainFrame = null;
	
	public EmptyWordsAction(ThemeEditor tE)
	{
	    mainFrame = tE;
	}
	/**
	   Mthode invoque lorsque l'action correspondante est active
	*/
	public void actionPerformed(ActionEvent e)
	{
	    new EmptyWordsDialog(mainFrame);
	}
    }
    
    /**
       Action "Configurer les options"
    */
    private class ConfigureOptionsAction implements ActionListener
    {
	private ThemeEditor mainFrame = null;
	private ThemeEditorVars teVars = null;
	public ConfigureOptionsAction(ThemeEditor tE, ThemeEditorVars tev)
	{
	    mainFrame = tE;
	    teVars = tev;
	}
	/**
	   Mthode invoque lorsque l'action correspondante est active
	*/
	public void actionPerformed(ActionEvent e)
	{
	    new ConfigOptionsDialog(mainFrame, true, teVars);
	}
    }
    
    /**
       Action "Index de l'aide"
    */
    private class HelpIndexAction implements ActionListener
    {
	ThemeEditor mainFrame = null;
	
	public HelpIndexAction(ThemeEditor tE)
	{
	    mainFrame = tE;
	}
	/**
	   Mthode invoque lorsque l'action correspondante est active
	*/
	public void actionPerformed(ActionEvent e)
	{
	    new HelpDialog(mainFrame);
	}
    }
    
    
    /**
       Action "A propos de l'application"
    */
    private class AboutAction implements ActionListener
    {
	ThemeEditor mainFrame = null;
	
	public AboutAction(ThemeEditor tE)
	{
	    mainFrame = tE;
	}
	/**
	   Mthode invoque lorsque l'action correspondante est active
	*/
	public void actionPerformed(ActionEvent e)
	{
	    new AboutDialog(mainFrame);
	}
    }
    
    /**
       Classe interne servant  grer la fermeture de l'application.
    */
    public class WindowClosing extends WindowAdapter
    {
	private ThemeEditor mainFrame = null;
	private ThemeEditorVars teVars = null;
	
	/**
	   Dfini une classe servant  grer la fermeture de l'application.
	 */
	public WindowClosing(ThemeEditor tE, ThemeEditorVars tev)
	{
	    mainFrame = tE;
	    teVars = tev;
	}
	
	/**
	   Mthode invoque lors de la fermeture de l'application
	*/
	public void windowClosing(WindowEvent e) 
	{
	    if (teVars.themeFileHasBeenModified())
		{
		    if (JOptionPane.showConfirmDialog(null, "Le fichier de thmes en cours d'dition n'a pas t sauvegard,\n souhaitez vous quand mme quitter l'application ?", "Quitter ThemeEditor", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) == JOptionPane.YES_OPTION)
			{
			    saveIniFile();
			    mainFrame.setVisible(false);
			    System.exit(0);
			}
		}
	    else
		{
		    saveIniFile();
		    mainFrame.setVisible(false);
		    System.exit(0);
		}
        }
    }
}
