/*
  Termin
*/

/******************************************************
ThemeEditor 1.0 est un diteur de thmes au format XML.
Copyright (C) 2000 David MASCLE et Damien BOUCHARD.
Auteurs: David MASCLE et Damien BOUCHARD.
*******************************************************/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.io.*;
import java.util.*;
import org.xml.sax.*;
import javax.xml.parsers.SAXParserFactory;  
import javax.xml.parsers.ParserConfigurationException;  
import javax.xml.parsers.SAXParser;  

/****************************************************************************************
<b>ThemeTree</b> est un diteur de thmes.<br><br>
<br><b>ThemeEditor 1.0</b> est un diteur de thmes au format XML.<br>
<br>Copyright &copy; 2000 David MASCLE et Damien BOUCHARD.<br>

@version : 1.0
@author : David MASCLE et Damien BOUCHARD

*****************************************************************************************/

public class ThemeTree extends JPanel
{
    // constante utilises pour la lecture et l'criture d'un fichier de thmes
    private static final String THEME_PROLOG = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\" standalone=\"yes\"?>\n";
    private static final String THEME_DTD = "<!DOCTYPE ThemeList\n[\n  <!ELEMENT ThemeList (Theme*)>\n  <!ELEMENT Theme (Definition, Word*)>\n  <!ELEMENT Definition (#PCDATA)>\n  <!ELEMENT Word (#PCDATA)>\n]>\n";
    private static final String THEMELIST_TAG = "ThemeList";
    private static final String THEME_TAG = "Theme";
    private static final String DEFINITION_TAG = "Definition";
    private static final String WORD_TAG = "Word";
    //*************************************************************************
    public DefaultMutableTreeNode rootNode;
    protected DefaultTreeModel treeModel;
    protected JTree tree;
    private Toolkit toolkit = Toolkit.getDefaultToolkit();
    private ThemeEditorVars teVars = null;
    private JPopupMenu popup = null;
    
    /**
       Mthode permettant le chargement du fichier de thmes.
    */
    public void load()
    {
	clear();
	ThemeTreeParser	tTreeParser = new ThemeTreeParser(this);
	tTreeParser.parse(teVars.getThemeFileName());
    }

    /**
       Mthode permettant l'enregistrement du fichier de thmes.
    */
    public void save()
    {
	PrintWriter outToFile = null;

	try
	    {
		outToFile = new PrintWriter(new BufferedWriter(new FileWriter(teVars.getThemeFileName())));
	    }
	catch (IOException e)
	    {
		System.err.println("Erreur d'entre/sortie sur le fichier de sortie : " + teVars.getThemeFileName());
		System.err.println("    " +  e.getLocalizedMessage());
	    }
	
	outToFile.println(THEME_PROLOG);
	outToFile.println(THEME_DTD);
	outToFile.println("<" + THEMELIST_TAG + ">");
	
	for (int iThemes = 0; iThemes < rootNode.getChildCount(); iThemes++)
	    {
		outToFile.println("\t<" + THEME_TAG + ">");
		TreeNode themeNode = rootNode.getChildAt(iThemes);
		outToFile.println("\t\t<" + DEFINITION_TAG + ">" + themeNode + "</" + DEFINITION_TAG + ">");
		for (int iWords = 0; iWords < themeNode.getChildCount(); iWords++)
		    {
			TreeNode wordNode = themeNode.getChildAt(iWords);
			outToFile.println("\t\t<" + WORD_TAG + ">" + wordNode + "</" + WORD_TAG + ">");
		    }
		outToFile.println("\t</" + THEME_TAG + ">");
	    }
	
	outToFile.println("</" + THEMELIST_TAG + ">");
	
	outToFile.flush();
	outToFile.close();
    }
    
    /**
       Dfini une zone d'dition d'une liste de thmes.
       @param teVars les variables de ThemeEditor.
    */
    public ThemeTree(ThemeEditorVars teVars)
    {
	this.teVars = teVars;
	// cration de l'arbre de visualisation
        rootNode = new DefaultMutableTreeNode("Liste des thmes");
        treeModel = new DefaultTreeModel(rootNode);
	// treeModel.addTreeModelListener(new MyTreeModelListener());
	tree = new JTree(treeModel);
        tree.setEditable(true);
        tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        tree.setShowsRootHandles(true);
	
        JScrollPane scrollPane = new JScrollPane(tree);
        setLayout(new GridLayout(1,0));
        add(scrollPane);
	
	// le menu popup
	JMenuItem menuItem = null;
	popup = new JPopupMenu();
	menuItem = new JMenuItem("Ajouter un thme");
	menuItem.addActionListener(new AddThemeAction());
	popup.add(menuItem);
	menuItem = new JMenuItem("Ajouter un mot");
	menuItem.addActionListener(new AddWordAction());
	popup.add(menuItem);
	popup.addSeparator();
	menuItem = new JMenuItem("Supprimer");
	menuItem.addActionListener(new DeleteAction());
	popup.add(menuItem);
	MouseListener popupListener = new PopupListener();
	tree.addMouseListener(popupListener);
    }
    
    /** 
	Supprime tous les noeuds sauf la racine.
    */
    public void clear()
    {
        rootNode.removeAllChildren();
        treeModel.reload();
    }

    /** 
	Supprime le noeud actuellement slectionn. 
    */
    public void removeCurrentNode()
    {
        TreePath currentSelection = tree.getSelectionPath();
        if (currentSelection != null) 
	    {
		DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode)(currentSelection.getLastPathComponent());
		MutableTreeNode parent = (MutableTreeNode)(currentNode.getParent());
		if (parent != null)
		    {
			treeModel.removeNodeFromParent(currentNode);
			return;
		    }
	    } 
        // Bip d'erreur dans le cas ou aucun noeud n'est slectionn ou si la racine est slectionne.
        toolkit.beep();
    }

    /**
       Mthode permettant l'ajout d'un thme dans la liste des thmes.
    */
    public void addTheme()
    {
	String answer = JOptionPane.showInputDialog(this, "Saisissez votre nouveau thme : ", "Ajout d'un nouveau thme", JOptionPane.QUESTION_MESSAGE);
	if (answer != null)
	    {
		DefaultMutableTreeNode nodeTemp = addTheme(rootNode, answer, true);
		teVars.setThemeFileIsModified(true);
	    }
    }
    
    /**
       Mthode permettant l'ajout d'un thme dans la liste des thmes  un endroit prcis.
       @param parent le noeud parent dans l'arbre.
       @param child le noeud  ajouter.
       @param shouldBeVisible boolen qui indique si le noeud ajouter doit etre visible.
    */
    public DefaultMutableTreeNode addTheme(DefaultMutableTreeNode parent, Object child, boolean shouldBeVisible)
    {
	DefaultMutableTreeNode childNode = null;
	if (parent == rootNode)
	    {
		childNode = new DefaultMutableTreeNode(new ThemeTreeNode(child.toString()));
		treeModel.insertNodeInto(childNode, parent, parent.getChildCount());
		if (shouldBeVisible)
		    tree.scrollPathToVisible(new TreePath(childNode.getPath()));
	    }
	return childNode;
    }
    
    /**
       Mthode retournant true si un mot est dj prsent dans le thme et false sinon.
       @param parent le noeud parent dans l'arbre.
       @param word le mot.
    */
    private boolean alreadyInTheme(DefaultMutableTreeNode parent, String word)
    {
	int iWords = 0;
	boolean found = false;
	DefaultMutableTreeNode wordNode = null;
	while ((iWords < parent.getChildCount()) && (!found))
	    {
		wordNode = (DefaultMutableTreeNode)parent.getChildAt(iWords);
		if (wordNode.toString().equals(word))
		    found = true;
		iWords++;
	    }
	return found;
    }

    /**
       Mthode permettant l'ajout d'un mot dans un thme.
    */
    public void addWord()
    {
	DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) tree.getSelectionPath().getLastPathComponent();

	if (parentNode.getUserObject() instanceof ThemeTreeNode)
	    {
		String answer = JOptionPane.showInputDialog(this, "Saisissez votre nouveau mot : ", "Ajout d'un nouveau mot", JOptionPane.QUESTION_MESSAGE);
		if (answer != null)
		    {
			if (alreadyInTheme(parentNode, answer))
			    {
				JOptionPane.showMessageDialog(this, "Le mot " + answer + " est dj prsent dans le thme !", "Erreur", JOptionPane.ERROR_MESSAGE);
			    }
			else 
			    {
				DefaultMutableTreeNode nodeTemp = addWord(parentNode, answer, true);
				if (teVars.getUseDatabase())
				    {
					ConnectDico cD = new ConnectDico(teVars.getMysqlServerAddress(), teVars.getMysqlServerPort(), teVars.getDatabaseName(), teVars.getUserName(), teVars.getUserPassword());
					java.util.List wordsList = cD.getWordsFromWord(answer);
					for (int index = 0; index < wordsList.size(); index++)
					    {
						if (!alreadyInTheme(parentNode, wordsList.get(index).toString()))
						    nodeTemp = addWord(parentNode, wordsList.get(index).toString() , true);
					    }
				    }
				teVars.setThemeFileIsModified(true);
			    }
		    }
	    }
	else JOptionPane.showMessageDialog(this, "Vous devez d'abord slectionner un thme pour pouvoir ajouter un mot !", "Erreur", JOptionPane.ERROR_MESSAGE); 
    }
    
    /**
       Mthode permettant l'ajout d'un mot pass en paramtre dans un thme avec vrification de la prsence avant ajout.
       @param theWord le mot.
    */
    public void addWord(String theWord) // addWord avec avec test de prsence et le mot en paramtre
    {
	try 
	    {
		DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) tree.getSelectionPath().getLastPathComponent();
		if (parentNode.getUserObject() instanceof ThemeTreeNode)
		    {		  
			if (alreadyInTheme(parentNode, theWord))
			    {
				JOptionPane.showMessageDialog(this, "Le mot " + theWord + " est dj prsent dans le thme !", "Erreur", JOptionPane.ERROR_MESSAGE);
			    }
			else 
			    {
				DefaultMutableTreeNode nodeTemp = addWord(parentNode,theWord , true);
				if (teVars.getUseDatabase())
				    {
					ConnectDico cD = new ConnectDico(teVars.getMysqlServerAddress(), teVars.getMysqlServerPort(), teVars.getDatabaseName(), teVars.getUserName(), teVars.getUserPassword());
					java.util.List wordsList = cD.getWordsFromWord(theWord);
					for (int index = 0; index < wordsList.size(); index++)
					    {
						if (!alreadyInTheme(parentNode, wordsList.get(index).toString()))
						    nodeTemp = addWord(parentNode, wordsList.get(index).toString() , true);
					    }
				    }
				teVars.setThemeFileIsModified(true);
			    }     	
		    }
		else JOptionPane.showMessageDialog(this, "Vous devez d'abord slectionner un thme pour pouvoir ajouter un mot !", "Erreur", JOptionPane.ERROR_MESSAGE); 
	    }
	catch ( NullPointerException ne)
	    {
		JOptionPane.showMessageDialog(this, "Vous devez d'abord slectionner un thme pour pouvoir ajouter un mot !", "Erreur", JOptionPane.ERROR_MESSAGE); 
	    }
    }
    
    /**
       Mthode permettant l'ajout d'un mot dans un thme prcis.
       @param parent le noeud parent dans l'arbre.
       @param child le noeud  ajouter.
       @param shouldBeVisible boolen qui indique si le noeud ajouter doit etre visible.
    */
    public DefaultMutableTreeNode addWord(DefaultMutableTreeNode parent, Object child, boolean shouldBeVisible)
    {
	DefaultMutableTreeNode childNode = null;
	Object parentClass = parent.getUserObject();
	if (parentClass instanceof ThemeTreeNode)
	    {
		childNode = new DefaultMutableTreeNode(new WordTreeNode(child.toString()));
		treeModel.insertNodeInto(childNode, parent, parent.getChildCount());
		if (shouldBeVisible)
		    tree.scrollPathToVisible(new TreePath(childNode.getPath()));
	    }
	return childNode;
    }
    
    /**
       Action "Supprimer"
    */
    private class DeleteAction implements ActionListener
    {
	/**
	   Mthode invoque lorsque l'action correspondante est active
	*/
	public void actionPerformed(ActionEvent e)
	{
	    removeCurrentNode();
	    teVars.setThemeFileIsModified(true);
	}
    }

    /**
       Action "Ajouter un thme"
    */
    private class AddThemeAction implements ActionListener
    {
	/**
	   Mthode invoque lorsque l'action correspondante est active
	*/
	public void actionPerformed(ActionEvent e)
	{
	    addTheme();
	}
    }
    
    /**
       Action "Ajouter un mot"
    */
    private class AddWordAction implements ActionListener
    {
	/**
	   Mthode invoque lorsque l'action correspondante est active
	*/
	public void actionPerformed(ActionEvent e)
	{
	    addWord();
	}
    }
    
    /**
       Classe interne grant l'affichage du menu contextuel permettant la modification de la liste des thmes.
    */
    class PopupListener extends MouseAdapter
    {
	public void mousePressed(MouseEvent e)
	{
	    maybeShowPopup(e);
	}
	
	public void mouseReleased(MouseEvent e)
	{
	    maybeShowPopup(e);
	}
	
	private void maybeShowPopup(MouseEvent e)
	{
	    if (e.isPopupTrigger())
		{
		    popup.show(e.getComponent(), e.getX(), e.getY());
		}
	}
    }
}

//**********************************************************************
//**********************************************************************
//**********************************************************************
//**********************************************************************
//**********************************************************************
//**********************************************************************

/****************************************************************************************************
<b>ThemeTreeParser</b> est un parseur XML de fichiers de thmes spcialis pour un ThemeTree.<br><br>
<br><b>ThemeEditor 1.0</b> est un diteur de thmes au format XML.<br>
<br>Copyright &copy; 2000 David MASCLE et Damien BOUCHARD.<br>

@version : 1.0
@author : David MASCLE et Damien BOUCHARD

 ***************************************************************************************************/

class ThemeTreeParser extends HandlerBase
{
    private ThemeTree themeTree = null;
    private DefaultMutableTreeNode currentThemeNodeTree = null;
    private static final int NO_TAG = 0;
    private static final int TAG_DEFINITION = 1;
    private static final int TAG_WORD = 2;    
    private int currentTagType = NO_TAG;

    /**
       Dfini un parseur XML de fichiers de thmes spcialis pour le ThemeTree.
       
       @param themeTree un arbre de thmes.
    */
    public ThemeTreeParser(ThemeTree themeTree)
    {
	this.themeTree = themeTree;
    }
    
    /**
       Cette mthode lance le parsiong d'un fichier de thmes.
       
       @param themeFileName un fichier de thmes.
    */
    public void parse (String themeFileName)
    {
	// nous utilisons un parseur avec validation
        SAXParserFactory factory = SAXParserFactory.newInstance();
        factory.setValidating(true);
        try 
	    {
		// nous parsons l'entre
		SAXParser saxParser = factory.newSAXParser();
		saxParser.parse( new File(themeFileName), new ThemeTreeParser(themeTree));
		
	    } 
	catch (SAXParseException spe) 
	    {
		// nous traitons les erreurs du parseur
		System.err.println ("\n** Erreur de parsing, ligne " + spe.getLineNumber() + ", uri " + spe.getSystemId());
		System.err.println("   " + spe.getMessage() );
		
		Exception  x = spe;
		if (spe.getException() != null)
		    x = spe.getException();
		x.printStackTrace();
	    } 
	catch (SAXException sxe) 
	    {
		// erreurs gnres par l'application
		Exception  x = sxe;
		if (sxe.getException() != null)
		    x = sxe.getException();
		x.printStackTrace();
	    }
	catch (ParserConfigurationException pce)
	    {
		// erreur de configuration du parseur
		pce.printStackTrace();
	    } 
	catch (IOException ioe) 
	    {
		// erreur d'entre/sortie
		ioe.printStackTrace();
	    }
    }
    
    //================================================
    // Surcharge des mthodes de la classe HandlerBase
    //================================================
    
    /**
       Pour plus de dtails consultez la documentation de JAXP.
    */
    public void startElement(String name, AttributeList attrs) throws SAXException
    {
	if (name.equals("Theme"))
	    {
		// nous ne faisons rien
	    }
	else if (name.equals("Definition"))
	    {
		currentTagType = TAG_DEFINITION;
	    }
	else if (name.equals("Word"))
	    {
		currentTagType = TAG_WORD;
	    }
	else currentTagType = NO_TAG;
    }
    
    /**
       Pour plus de dtails consultez la documentation de JAXP.
    */
    public void endElement(String name) throws SAXException
    {
	if (name.equals("Theme"))
	    {
		// nous ne faisons rien
	    }
	else if (name.equals("Definition"))
	    {
		currentTagType = NO_TAG;
	    }
	else if (name.equals("Word"))
	    {
		currentTagType = NO_TAG;
	    }
	else currentTagType = NO_TAG;
    }

    /**
       Pour plus de dtails consultez la documentation de JAXP.
    */
    public void characters(char buf [], int offset, int len) throws SAXException
    {
	String s = new String(buf, offset, len);
	if (currentTagType == TAG_DEFINITION)
	    {
		//tempoTheme.setDefinition(s);
		currentThemeNodeTree =themeTree.addTheme(themeTree.rootNode, s, true);
	    }
	else if (currentTagType == TAG_WORD)
	    {
		//tempoTheme.addWord(s);
		themeTree.addWord(currentThemeNodeTree, s, false);
	    }
    }

    //===========================================
    // mthodes de gestion des erreurs du parseur
    //===========================================

    /**
      Pour plus de dtails consultez la documentation de JAXP.
    */
    public void error(SAXParseException e) throws SAXParseException
    {
        throw e;
    }

    /**
      Pour plus de dtails consultez la documentation de JAXP.
    */
    public void warning(SAXParseException err) throws SAXParseException
    {
        System.err.println ("** Warning, ligne " + err.getLineNumber() + ", uri " + err.getSystemId());
        System.err.println("   " + err.getMessage());
    }
}
