/*
  Termin
*/

/******************************************************
ThemeEditor 1.0 est un diteur de thmes au format XML.
Copyright (C) 2000 David MASCLE et Damien BOUCHARD.
Auteurs: David MASCLE et Damien BOUCHARD.
*******************************************************/

import java.io.*;
import java.util.*;
import org.xml.sax.*;
import javax.xml.parsers.SAXParserFactory;  
import javax.xml.parsers.ParserConfigurationException;  
import javax.xml.parsers.SAXParser;  

/*****************************************************************************
<b>XmlThemeParser</b> est un parseur XML de fichiers de thmes.<br><br>
<br><b>ThemeEditor 1.0</b> est un diteur de thmes au format XML.<br>
<br>Copyright &copy; 2000 David MASCLE et Damien BOUCHARD.<br>

@version : 1.0
@author : David MASCLE et Damien BOUCHARD

******************************************************************************/


public class XmlThemeParser extends HandlerBase
{
    private List themeList = null;

    private Theme tempoTheme = null;

    private static final int NO_TAG = 0;
    private static final int TAG_DEFINITION = 1;
    private static final int TAG_WORD = 2;

    private int currentTagType = NO_TAG;

    /**
      Dfini un parseur XML de fichiers de thmes.

      @param themeList une liste de thmes.
    */
    public XmlThemeParser(List themeList)
    {
	this.themeList = themeList;
    }
    
    /**
      Cette mthode lance le parsiong d'un fichier de thmes.
      
      @param themeFileName un fichier de thmes.
    */
    public void parse (String themeFileName)
    {
	// nous utilisons un parseur avec validation
        SAXParserFactory factory = SAXParserFactory.newInstance();
        factory.setValidating(true);
        try 
	    {
		// nous parsons l'entre
		SAXParser saxParser = factory.newSAXParser();
		saxParser.parse( new File(themeFileName), new XmlThemeParser(themeList));
		
	    } 
	catch (SAXParseException spe) 
	    {
		// nous traitons les erreurs du parseur
		System.err.println ("\n** Erreur de parsing, ligne " + spe.getLineNumber() + ", uri " + spe.getSystemId());
		System.err.println("   " + spe.getMessage() );

		Exception  x = spe;
		if (spe.getException() != null)
		    x = spe.getException();
		x.printStackTrace();
	    } 
	catch (SAXException sxe) 
	    {
		// erreurs gnres par l'application
		Exception  x = sxe;
		if (sxe.getException() != null)
		    x = sxe.getException();
		x.printStackTrace();
	    }
	catch (ParserConfigurationException pce)
	    {
		// erreur de configuration du parseur
		pce.printStackTrace();
	    } 
	catch (IOException ioe) 
	    {
		// erreur d'entre/sortie
		ioe.printStackTrace();
	    }
    }

    //================================================
    // Surcharge des mthodes de la classe HandlerBase
    //================================================
    
    /**
      Pour plus de dtails consultez la documentation de JAXP.
    */
    public void startElement(String name, AttributeList attrs) throws SAXException
    {
	if (name.equals("Theme"))
	    {
		tempoTheme = new Theme("");
	    }
	else if (name.equals("Definition"))
	    {
		currentTagType = TAG_DEFINITION;
	    }
	else if (name.equals("Word"))
	    {
		currentTagType = TAG_WORD;
	    }
	else currentTagType = NO_TAG;
    }

    /**
      Pour plus de dtails consultez la documentation de JAXP.
    */
    public void endElement(String name) throws SAXException
    {
	if (name.equals("Theme"))
	    {
		themeList.add(tempoTheme);
		tempoTheme = null;
	    }
	else if (name.equals("Definition"))
	    {
		currentTagType = NO_TAG;
	    }
	else if (name.equals("Word"))
	    {
		currentTagType = NO_TAG;
	    }
	else currentTagType = NO_TAG;
    }

    /**
      Pour plus de dtails consultez la documentation de JAXP.
    */
    public void characters(char buf [], int offset, int len) throws SAXException
    {
	String s = new String(buf, offset, len);
        if (currentTagType == TAG_DEFINITION)
	    {
		tempoTheme.setDefinition(s);
	    }
	else if (currentTagType == TAG_WORD)
	    {
		tempoTheme.addWord(s);
	    }
    }

    //===========================================
    // mthodes de gestion des erreurs du parseur
    //===========================================
    
    /**
      Pour plus de dtails consultez la documentation de JAXP.
    */
    public void error(SAXParseException e) throws SAXParseException
    {
        throw e;
    }

    /**
      Pour plus de dtails consultez la documentation de JAXP.
    */
    public void warning(SAXParseException err) throws SAXParseException
    {
        System.err.println ("** Warning, ligne " + err.getLineNumber() + ", uri " + err.getSystemId());
        System.err.println("   " + err.getMessage());
    }
}
